"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Emitter_1 = __importDefault(require("licia/Emitter"));
const _1 = __importDefault(require("licia/$"));
const util_1 = require("./util");
const each_1 = __importDefault(require("licia/each"));
const extend_1 = __importDefault(require("licia/extend"));
const defaults_1 = __importDefault(require("licia/defaults"));
const remove_1 = __importDefault(require("licia/remove"));
class Component extends Emitter_1.default {
    constructor(container, { compName }, { theme = 'light' } = {}) {
        super();
        this.subComponents = [];
        this.compName = compName;
        this.c = (0, util_1.classPrefix)(compName);
        this.options = {};
        this.container = container;
        this.$container = (0, _1.default)(container);
        this.$container.addClass([
            `luna-${compName}`,
            this.c(`platform-${(0, util_1.getPlatform)()}`),
        ]);
        this.on('optionChange', (name, val, oldVal) => {
            const c = this.c;
            if (name === 'theme') {
                this.$container
                    .rmClass(c(`theme-${oldVal}`))
                    .addClass(c(`theme-${val}`));
                (0, each_1.default)(this.subComponents, (component) => component.setOption('theme', val));
            }
        });
        this.setOption('theme', theme);
    }
    destroy() {
        this.destroySubComponents();
        const { c } = this;
        this.$container
            .rmClass(`luna-${this.compName}`)
            .rmClass(c(`platform-${(0, util_1.getPlatform)()}`))
            .rmClass(c(`theme-${this.options.theme}`));
        this.$container.html('');
        this.emit('destroy');
        this.removeAllListeners();
    }
    setOption(name, val) {
        const options = this.options;
        let newOptions = {};
        if (typeof name === 'string') {
            newOptions[name] = val;
        }
        else {
            newOptions = name;
        }
        (0, each_1.default)(newOptions, (val, name) => {
            const oldVal = options[name];
            options[name] = val;
            this.emit('optionChange', name, val, oldVal);
        });
    }
    getOption(name) {
        return this.options[name];
    }
    addSubComponent(component) {
        component.setOption('theme', this.options.theme);
        this.subComponents.push(component);
    }
    removeSubComponent(component) {
        (0, remove_1.default)(this.subComponents, (com) => com === component);
    }
    destroySubComponents() {
        (0, each_1.default)(this.subComponents, (component) => component.destroy());
        this.subComponents = [];
    }
    initOptions(options, defs = {}) {
        (0, defaults_1.default)(options, defs);
        (0, extend_1.default)(this.options, options);
    }
    find(selector) {
        return this.$container.find(this.c(selector));
    }
}
exports.default = Component;
