"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Component_1 = __importDefault(require("../share/Component"));
const core_1 = __importDefault(require("highlight.js/lib/core"));
const javascript_1 = __importDefault(require("highlight.js/lib/languages/javascript"));
const xml_1 = __importDefault(require("highlight.js/lib/languages/xml"));
const css_1 = __importDefault(require("highlight.js/lib/languages/css"));
const last_1 = __importDefault(require("licia/last"));
const trim_1 = __importDefault(require("licia/trim"));
const isEmpty_1 = __importDefault(require("licia/isEmpty"));
const each_1 = __importDefault(require("licia/each"));
const startWith_1 = __importDefault(require("licia/startWith"));
const throttle_1 = __importDefault(require("licia/throttle"));
const copy_1 = __importDefault(require("licia/copy"));
core_1.default.configure({
    classPrefix: 'luna-syntax-highlighter-',
});
core_1.default.registerLanguage('javascript', javascript_1.default);
core_1.default.registerLanguage('css', css_1.default);
core_1.default.registerLanguage('xml', xml_1.default);
class SyntaxHighlighter extends Component_1.default {
    constructor(container, options = {}) {
        super(container, { compName: 'syntax-highlighter' }, options);
        this.copy = () => {
            const { c } = this;
            (0, copy_1.default)(this.options.code);
            const $icon = this.$copy.find(c('.icon'));
            $icon.addClass(c('icon-check')).rmClass(c('icon-copy'));
            setTimeout(() => {
                $icon.rmClass(c('icon-check')).addClass(c('icon-copy'));
            }, 1000);
        };
        this.updateCopyPos = () => {
            const { container } = this;
            this.$copy.css({
                top: container.scrollTop + 5,
                right: -container.scrollLeft + 5,
            });
        };
        let code = '';
        let language = 'javascript';
        if (!options.code && container.tagName === 'PRE') {
            const $code = this.$container.find('code');
            if (!(0, isEmpty_1.default)($code)) {
                code = $code.text();
                const className = $code.attr('class') || '';
                if ((0, startWith_1.default)(className, 'language-')) {
                    language = className.replace('language-', '');
                }
            }
        }
        this.initOptions(options, {
            code,
            language,
            showLineNumbers: true,
            wrapLongLines: true,
        });
        this.render = (0, throttle_1.default)(() => this._render(), 16);
        this.initTpl();
        this.$code = this.find('.code');
        this.$copy = this.find('.copy');
        if (this.options.code) {
            this.render();
        }
        this.bindEvent();
    }
    destroy() {
        this.$container.off('scroll', this.updateCopyPos);
        super.destroy();
    }
    initTpl() {
        this.$container.html(this.c(`<code class="code"></code><div class="copy"><span class="icon icon-copy"></span></div>`));
    }
    bindEvent() {
        this.on('optionChange', () => this.render());
        this.$container.on('scroll', this.updateCopyPos);
        this.$copy.on('click', this.copy);
    }
    _render() {
        const { c, $copy, $code } = this;
        const { code, language, showLineNumbers, wrapLongLines } = this.options;
        if (wrapLongLines) {
            $code.addClass(c('wrap-long-lines'));
        }
        else {
            $code.rmClass(c('wrap-long-lines'));
        }
        const highlightCode = core_1.default.highlight(code, { language }).value;
        if (!showLineNumbers) {
            $code.rmClass(c('line-numbers'));
            return this.$code.html(highlightCode);
        }
        $code.addClass(c('line-numbers'));
        let lines = getLines(highlightCode);
        if ((0, isEmpty_1.default)(lines)) {
            lines = ['&nbsp;'];
        }
        if (lines.length > 3) {
            $copy.show();
        }
        else {
            $copy.hide();
        }
        if (!(0, trim_1.default)((0, last_1.default)(lines))) {
            lines.pop();
        }
        let body = '';
        (0, each_1.default)(lines, (line, idx) => {
            body += `<div class="${c('table-row')}"><div class="${c('line-number')}">${idx + 1}</div><div class="${c('line-code')}">${line || ' '}</div></div>`;
        });
        $code.html(`<div class="${c('table')}">${body}</div>`);
        this.updateCopyPos();
    }
    static registerLanguage(name, fn) {
        core_1.default.registerLanguage(name, fn);
    }
    static getLanguage(name) {
        return core_1.default.getLanguage(name);
    }
}
exports.default = SyntaxHighlighter;
const regBreakLine = /\r\n|\r|\n/g;
function getLines(code) {
    if (code.length === 0) {
        return [];
    }
    return code.split(regBreakLine);
}
module.exports = SyntaxHighlighter;
module.exports.default = SyntaxHighlighter;
